<?php

namespace Nest;

use Throwable;

/**
 * Default exception handler for Nest application.
 * Support multiple sub-handlers, typically added by engines.
 */
class ExceptionHandler
{
	/**
	 * @var (callable(Throwable): void)[]
	 */
	protected array $handlers = [];

	/**
	 * Run all exception handlers.
	 * @param Throwable $exception The exception.
	 * @return void
	 */
	public function __invoke(Throwable $exception): void
	{
		foreach ($this->handlers as $handler)
		{ // Execute each registered exception handler.
			$handler($exception);
		}
	}

	/**
	 * Add an exception handler.
	 * @param callable(Throwable): void $handler The exception handler function.
	 * @return $this
	 */
	public function add(callable $handler): static
	{
		$this->handlers[] = $handler;
		return $this;
	}
}
