<?php

namespace Nest\Utils;

use Nest\Exceptions\InvalidTypeException;

/**
 * Extract the class name from its full name (\Namespace\Classname).
 * @param string $class_fullname Full name of the class, with its namespace.
 * @return string The class name.
 */
function get_class_name_from_fullname(string $class_fullname): string
{ // Keep last part of the string, after the last '\'.
	return substr($class_fullname, mb_strrpos($class_fullname, "\\") + 1);
}

/**
 * Check that the given object or class is of the expected type.
 * @template T
 * @param string|object $object_or_class Object or class to check.
 * @param class-string<T> $expected_type Expected class type.
 * @return true
 * @throws InvalidTypeException
 */
function expect_type(mixed $object_or_class, string $expected_type): true
{
	if (!is_a($object_or_class, $expected_type))
		throw new InvalidTypeException($expected_type, $object_or_class);
	return true;
}

/**
 * Get all classes of an object or a class.
 * @param string|object $object_or_class
 * @return string[] All classes of an object or a class (full inheritance tree).
 */
function get_all_classes(mixed $object_or_class): array
{
	// Get the class of the current object.
	if (!is_string($object_or_class))
		$object_or_class = get_class($object_or_class);

	// Try to get the parent class of the current class.
	$parentClass = get_parent_class($object_or_class);

	// Merge classes list, preprending with the current class.
	return [$object_or_class, ...($parentClass ? get_all_classes($parentClass) : [])];
}
